#!/bin/bash

# Pfade anpassen
TEXTURE_DIR="assets/minecraft/textures/item/custom"
MODEL_DIR="assets/minecraft/models/item"
CUSTOM_MODEL_DIR="assets/minecraft/models/custom_items"
YML_FILE="custom_models.yml"

# Funktion zum Formatieren von Item-Namen für übergeordnete Einträge
format_parent_name() {
    echo "$1" | tr ' ' '_' | tr '[:lower:]' '[:upper:]'
}

# Funktion zum Formatieren von Item-Namen für untergeordnete Einträge
format_item_name() {
    echo "$1" | tr '_' ' ' | awk '{for (i=1; i<=NF; i++) $i=toupper(substr($i,1,1)) tolower(substr($i,2))} 1'
}

# YAML-Datei initialisieren
echo "# Custom Models" > "$YML_FILE"

# Temporäre Datei zur Sammlung der YAML-Einträge
TEMP_YML_FILE=$(mktemp)

# Assoziatives Array zur Verfolgung bereits verarbeiteter Texturenkombinationen je Vanilla Item
declare -A PROCESSED_TEXTURE_COMBINATIONS

# Durch alle vanilla item Ordner gehen
for VANILLA_ITEM_DIR in "$TEXTURE_DIR"/*; do
    if [ -d "$VANILLA_ITEM_DIR" ]; then
        VANILLA_ITEM=$(basename "$VANILLA_ITEM_DIR")
        VANILLA_ITEM_UPPER=$(echo "$VANILLA_ITEM" | tr '[:lower:]' '[:upper:]')

        # Pfad zur vanilla item JSON-Datei
        VANILLA_ITEM_JSON="$MODEL_DIR/$VANILLA_ITEM.json"

        # Prüfen, ob die vanilla item JSON-Datei existiert
        if [ -f "$VANILLA_ITEM_JSON" ]; then
            # Backup der originalen JSON-Datei
            cp "$VANILLA_ITEM_JSON" "$VANILLA_ITEM_JSON.bak"

            # Sicherstellen, dass die overrides Liste existiert
            jq '.overrides = (.overrides // [])' "$VANILLA_ITEM_JSON.bak" > "$VANILLA_ITEM_JSON"
        else
            # Falls die JSON-Datei nicht existiert, wird sie erstellt
            echo '{"overrides": []}' > "$VANILLA_ITEM_JSON"
        fi

        # Alle möglichen Texturen sammeln
        CUSTOM_TEXTURES=("$VANILLA_ITEM_DIR"/*.png "$VANILLA_ITEM_DIR"/*_side.png "$VANILLA_ITEM_DIR"/*_top.png "$VANILLA_ITEM_DIR"/*_bottom.png)

        # Assoziatives Array zur Verfolgung bereits verarbeiteter Texturen je Vanilla Item
        declare -A PROCESSED_TEXTURES

        # Durch alle png Dateien im Ordner gehen
        COUNTER=1
        for CUSTOM_TEXTURE_PNG in "${CUSTOM_TEXTURES[@]}"; do
            if [ -f "$CUSTOM_TEXTURE_PNG" ]; then
		CUSTOM_TEXTURE=$(basename "$CUSTOM_TEXTURE_PNG" .png)

                # Prüfen, ob diese Texturenkombination bereits verarbeitet wurde
                if [ "${PROCESSED_TEXTURES[$CUSTOM_TEXTURE]}" ]; then
                    continue  # Weiter zum nächsten, wenn bereits verarbeitet
                fi

		if [[ "$CUSTOM_TEXTURE" =~ _top$ ]]; then
                    CUSTOM_MODEL_JSON="${CUSTOM_TEXTURE%_top}.json"
		    CUSTOM_TEXTURE_TOP="$CUSTOM_TEXTURE"
		    CUSTOM_TEXTURE_SIDE="${CUSTOM_TEXTURE/_top/_side}"
		    CUSTOM_TEXTURE_BOTTOM="${CUSTOM_TEXTURE/_top/_bottom}"
		elif [[ "$CUSTOM_TEXTURE" =~ _side$ ]]; then
                    CUSTOM_MODEL_JSON="${CUSTOM_TEXTURE%_side}.json"
		    CUSTOM_TEXTURE_TOP="${CUSTOM_TEXTURE/_side/_top}"
		    CUSTOM_TEXTURE_SIDE="$CUSTOM_TEXTURE"
		    CUSTOM_TEXTURE_BOTTOM="${CUSTOM_TEXTURE/_side/_bottom}"
		elif [[ "$CUSTOM_TEXTURE" =~ _sides$ ]]; then
                    CUSTOM_MODEL_JSON="${CUSTOM_TEXTURE%_sides}.json"
		    CUSTOM_TEXTURE_TOP="${CUSTOM_TEXTURE/_sides/_top}"
		    CUSTOM_TEXTURE_SIDE="$CUSTOM_TEXTURE"
		    CUSTOM_TEXTURE_BOTTOM="${CUSTOM_TEXTURE/_sides/_bottom}"
		elif [[ "$CUSTOM_TEXTURE" =~ _bottom$ ]]; then
                    CUSTOM_MODEL_JSON="${CUSTOM_TEXTURE%_bottom}.json"
		    CUSTOM_TEXTURE_TOP="${CUSTOM_TEXTURE/_bottom/_top}"
		    CUSTOM_TEXTURE_SIDE="${CUSTOM_TEXTURE/_bottom/_side}"
		    CUSTOM_TEXTURE_BOTTOM="$CUSTOM_TEXTURE"
		elif [[ "$CUSTOM_TEXTURE" =~ _back$ ]]; then
                    continue
		else
                    CUSTOM_MODEL_JSON="$CUSTOM_TEXTURE.json"
		    CUSTOM_TEXTURE_TOP="$CUSTOM_TEXTURE"
		    CUSTOM_TEXTURE_SIDE="$CUSTOM_TEXTURE"
		    CUSTOM_TEXTURE_BOTTOM="$CUSTOM_TEXTURE"
		fi
		
		if [[ ! -f "$VANILLA_ITEM_DIR/$CUSTOM_TEXTURE_TOP.png" ]]; then
		    CUSTOM_TEXTURE_TOP="${CUSTOM_TEXTURE_TOP%_top}"
		fi
		if [[ ! -f "$VANILLA_ITEM_DIR/$CUSTOM_TEXTURE_SIDE.png" ]]; then
		    CUSTOM_TEXTURE_SIDE="${CUSTOM_TEXTURE_SIDE%_side}"
		fi
		CUSTOM_TEXTURE_FRONT="$CUSTOM_TEXTURE_SIDE"
		if [[ ! -f "$VANILLA_ITEM_DIR/$CUSTOM_TEXTURE_BOTTOM.png" ]]; then
		    CUSTOM_TEXTURE_BOTTOM="${CUSTOM_TEXTURE_BOTTOM%_bottom}"
		    if [[ -f "$VANILLA_ITEM_DIR/$CUSTOM_TEXTURE_TOP.png" ]]; then
		    	CUSTOM_TEXTURE_BOTTOM="$CUSTOM_TEXTURE_TOP"
			if [[ -f "$VANILLA_ITEM_DIR/${CUSTOM_TEXTURE_TOP/_top/_back}.png" ]]; then
				CUSTOM_TEXTURE_SIDE="${CUSTOM_TEXTURE_TOP/_top/_back}"
		    	fi
		    fi
		fi
		
		
		
		
                # Custom Model JSON Datei erstellen
                mkdir -p "$(dirname "$CUSTOM_MODEL_DIR/$CUSTOM_MODEL_JSON")"
                cat <<EOF > "$CUSTOM_MODEL_DIR/$CUSTOM_MODEL_JSON"
{
    "parent": "minecraft:block/cube_all",
    "textures": {
        "0": "item/custom/$VANILLA_ITEM/${CUSTOM_TEXTURE_TOP}",
        "1": "item/custom/$VANILLA_ITEM/${CUSTOM_TEXTURE_FRONT}",
        "2": "item/custom/$VANILLA_ITEM/${CUSTOM_TEXTURE_SIDE}",
        "3": "item/custom/$VANILLA_ITEM/${CUSTOM_TEXTURE_BOTTOM}"
    },
    "elements": [
        {
            "from": [0, 0, 0],
            "to": [16, 16, 16],
            "faces": {
                "north": {"texture": "#1"},
                "east": {"texture": "#2"},
                "south": {"texture": "#2"},
                "west": {"texture": "#2"},
                "up": {"texture": "#0"},
                "down": {"texture": "#3"}
            }
        }
    ]
}
EOF

                # Override in vanilla item JSON hinzufügen
                if [ -f "$VANILLA_ITEM_JSON" ]; then
                    jq --arg model "custom_items/${CUSTOM_MODEL_JSON%.json}" \
                       --argjson predicate "{\"custom_model_data\": $COUNTER}" \
                       '.overrides += [{"predicate": $predicate, "model": $model}]' \
                       "$VANILLA_ITEM_JSON" > "$VANILLA_ITEM_JSON.tmp" && mv "$VANILLA_ITEM_JSON.tmp" "$VANILLA_ITEM_JSON"
                    
                    # Markiere diese Texturenkombination als verarbeitet
                    PROCESSED_TEXTURE_COMBINATIONS[${CUSTOM_MODEL_JSON%.json}]=1
                fi

                # Eintrag in temporäre YAML-Datei hinzufügen, wenn nicht bereits vorhanden
                if [ ! "${PROCESSED_TEXTURES[${CUSTOM_MODEL_JSON%.json}]}" ]; then
                    FORMATTED_PARENT_NAME=$(format_parent_name "$VANILLA_ITEM_UPPER")
                    FORMATTED_ITEM_NAME=$(format_item_name "${CUSTOM_MODEL_JSON%.json}")
                    echo "$FORMATTED_PARENT_NAME:" >> "$TEMP_YML_FILE"
                    echo "  '$COUNTER': $FORMATTED_ITEM_NAME" >> "$TEMP_YML_FILE"

                    # Markiere diese Texturen als verarbeitet
                    PROCESSED_TEXTURES[$CUSTOM_TEXTURE_TOP]=1
                    PROCESSED_TEXTURES[$CUSTOM_TEXTURE_SIDE]=1
                    PROCESSED_TEXTURES[$CUSTOM_TEXTURE_BOTTOM]=1

                    # COUNTER erhöhen
                    COUNTER=$((COUNTER + 1))
                fi
            fi
        done
    fi
done

# Doppelte Einträge zusammenführen und richtig formatieren
awk '/^#/ {print; next} {if($0 ~ /:$/){item=$0} else {sub(/^  /, ""); entry[item]=entry[item] "  " $0 "\n"}} END {for (i in entry) {print i; printf "%s", entry[i]}}' "$TEMP_YML_FILE" >> "$YML_FILE"

# Temporäre Datei löschen
rm "$TEMP_YML_FILE"

